UnicaenDbImport
===============

 * [Introduction](#introduction)
 * [La différence avec le module UnicaenImport ?](#la-différence-avec-le-module-unicaenimport-)
 * [Installation](#installation)
 * [Configuration](#configuration)
 * [Utilisation](#utilisation)
 * [Contraintes](#contraintes)
 * [Fonctionnement](#fonctionnement)
 * [Exemples](#exemples)
   1. [Import d'une table de données distantes](#exemple-1-importation-dune-table-de-données-distantes)
   2. [Import d'un select de données distantes](#exemple-2-importation-dun-select-de-données-distantes)
   3. [Synchronisation de données locales](#exemple-3-synchronisation-de-données-locales)
   4. [Synchronisation de données distantes](#exemple-4-synchronisation-de-données-distantes)
   5. [Synchronisation de données distantes avec récupération des clés étrangères](#exemple-5-synchronisation-de-données-distantes-avec-récupération-des-clés-étrangères)
   6. [[Bonus] Appliquer un CRON](#exemple-z-appliquer-un-cron)
 
*NB: Refonte en cours, plus d'information sur l'ancien moteur (peut être encore actuel) d'UnicaenDbImport en [cliquant ici](https://git.unicaen.fr/lib/unicaen/db-import/tree/2ceb3a7e#dans-le-moteur).*
 
 
Introduction
------------

Ce module réalise l'import et/ou la synchronisation de données *sources* vers une table d'une base de données 
*destination*. 

La source peut être :
- soit une base de données (une table ou un "select"), 
- soit une API (web service).

Principe de l'*import* :
  - La table destination est vidée systématiquement (truncate).
  - Les données obtenues de la source sont insérées dans la table destination.

Principe de la *synchronisation* :

  - Si la source contient un enregistrement qui n'existe pas dans la destination, il est ajouté dans cette dernière.
  - Si la source contient un enregistrement qui existe aussi dans la destination avec les mêmes valeurs de colonnes, 
    rien n'est fait.
  - Si la source contient un enregistrement qui existe aussi dans la destination mais avec des valeurs de colonnes 
    différentes, les colonnes de la destination sont mises à jour en conséquence.
  - Si la source ne contient pas un enregistrement qui existe dans la destination, l'enregistrement destination est historisé 
    (i.e. marqué "supprimé").
  - Si la source contient un enregistrement qui existe aussi dans la destination mais marqué "supprimé", ce dernier est 
    dé-historisé.

Les données source et les enregistrements destination doivent avoir un identifiant unique en commun permettant 
de les rapprocher : on l'appellera "code source" (cf. paramètre de config `source_code_column`). 
**Cet identifiant DOIT être de type chaîne de caractères.**


La différence avec le module UnicaenImport ?
--------------------------------------------
  
Comme son nom ne l'indique pas, UnicaenImport fonctionne uniquement entre 2 bases de données **Oracle**.

UnicaenDbImport fonctionne en majeure partie en PHP, tout en déléguant au maximum au SGBD ce qu'il sait bien faire 
(`select FULL OUTER JOIN` pour le différentiel entre données source et destination, par exemple). 
Par conséquent, il est possible et souhaitable de l'enrichir pour implémenter un import/synchronisation vers 
différentes plateformes de base de données destination. 
Les plateformes de base de données destination suivantes sont implémentées :
  - PostgreSQL

*NB: En revanche toutes les plateformes de base de données source sont supportées car on se contente d'y faire un select 
pour alimenter une table intermédiaire.*
 
 
Installation
------------
 
     composer require unicaen/db-import


Configuration
-------------

- Récupérer les fichiers config de distribution :
```bash
cp -n vendor/unicaen/db-import/config/unicaen-db-import.global.php.dist config/autoload/unicaen-db-import.global.php
cp -n vendor/unicaen/db-import/config/unicaen-db-import.local.php.dist config/autoload/unicaen-db-import.local.php
```

- Adapter leur contenu à vos besoins.

- Configuration nécessaire en cas d'utilisation du module unicaen/bjy-authorize :

```php
'bjyauthorize'    => [
    'guards' => [
        PrivilegeController::class => [
            [
                'controller' => 'UnicaenDbImport\Controller\Console',
                'action'     => [
                    'runImport',
                    'runSynchro',
                ],
                'roles' => [],
            ],
        ],
    ],
],
```

Utilisation
-----------

Le module possède deux mécanismes distincts :
1. import : réalise un import "brut" des données, autrement dit une copie, d'une source vers une destination;
2. synchro : réalise une synchronisation entre une source et une destination en gérant une historisation (created_on, updated_on, deleted_on).

Le module fournit donc une ligne de commande pour lancer :

  - un import par son nom :
  
    ```bash
    php public/index.php run import --name "NOM_ETABLI_DANS_LA_CONFIG"
    ```
    
  - tous les imports :
  
    ```bash
    php public/index.php run import --all
    ```
    
  - une synchro par son nom :
    
    ```bash
    php public/index.php run synchro --name "NOM_ETABLI_DANS_LA_CONFIG"
    ```
      
  - toutes les synchros :
    
    ```bash
    public/index.php run synchro --all
    ```

*NB: L'exécution d'une de ces commandes n'est effectif qu'une fois. Pour importer/synchroniser en permanence (dans) 
la base de données destination, il faut programmer le lancement périodique de cette commande à l'aide de CRON.* 
Exemple : `*/15 6-19 * * 1-5 root /usr/bin/php /path/to/app/public/index.php run import --all 1> /tmp/zebu-cron.log 2>&1`


Contraintes
-----------

### Identifiant commun

L'identifiant unique commun des enregistrements (`source_code_column`) source et destination doit être de type 
**chaîne de caractères**.

### Table destination

La *table destination* doit obligatoirement posséder les colonnes d'historique `created_on`, `updated_on` 
et `deleted_on`. Exemple avec PostgreSQL :
```sql
ALTER TABLE TABLE_DESTINATION ADD COLUMN created_on TIMESTAMP(0) WITH TIME ZONE DEFAULT LOCALTIMESTAMP(0) NOT NULL;
ALTER TABLE TABLE_DESTINATION ADD COLUMN updated_on TIMESTAMP(0) WITH TIME ZONE;
ALTER TABLE TABLE_DESTINATION ADD COLUMN deleted_on TIMESTAMP(0) WITH TIME ZONE;
```


Fonctionnement
--------------

### Import de données

Afin de s'adapter au plus grand nombre de SGBD, UnicaenDbImport s'appuie sur l'_ORM Doctrine 2_.
Ainsi nous effectuons le déroulement suivant :

<!--Voir le dossier documentation/ pour toutes modifications-->
![fonctionnement_import.png](documentation/fonctionnement_import.png)

*Schématisation du déroulement d'un import*

1. Suppression des données existantes dans _Destination_
2. Récupération des données de la _Source_
3. Génération des requêtes SQL (insert, update, ...) depuis PHP
4. Exécution des requêtes / Récupération des données vers _Destination_


### Synchronisation de données locales

L'un des mécanismes d'UnicaenDbImport est d'appliquer un système de synchronisation avec gestion d'une historisation.
Avec une table _Source_ et une table _Destination_ toutes deux dans une base locale, le déroulement est le suivant :

<!--Voir le dossier documentation/ pour toutes modifications-->
![fonctionnement_synchro_locale.png](documentation/fonctionnement_synchro_locale.png)

*Schématisation du déroulement d'une synchro (locale)*

1. Création d'une vue différentielle
2. Préparation/détection des actions à appliquer (INSERT, UPDATE, ...)
3. Application des mises à jour (avec historisation : created_on, updated_on, ...)


### Synchronisation de données distantes

Concernant le mécanisme de synchronisation, il est possible de spécifier une _Source_ de type table/select provenant 
d'une base distante, ou de type API (web service).
Dans ce cas, les mécanismes d'[import](#import-de-données) et de [synchro (locale)](#synchronisation-de-données-locales) 
seront réalisés successivement.

<!--Voir le dossier documentation/ pour toutes modifications-->
![fonctionnement_synchro_distant.png](documentation/fonctionnement_synchro_distant.png)

*Schématisation du déroulement d'une synchro (distant)*

1. Déclenchement du mécanisme d'import de la _Source_ vers une table _Temporaire_
2. Déclenchement du mécanisme de synchro entre la table _Temporaire_ et la table _Destination
3. Suppression de la table _Temporaire_


Exemples
--------

Pour les exemples suivants, on suppose posséder :
 * 1 Base de donnée **A** locale (celle de notre application) contenant les tables :
   * **UTILISATEUR**(int ID, str CODE, str NOM, str PRENOM, date NAISSANCE)
   * **AUTRE_UTILISATEUR**(str CODE, str NOM, str PRENOM, date NAISSANCE)
   * **COMPOSANTE**(int ID, str CODE, str NOM)
   * **FORMATION**(int ID, str CODE, str NOM, str COMPOSANTE_ID)
   * **PAIN_AU_CHOCOLAT**(int ID, str NOM, str BOULANGERIE)
 * 1 Base de donnée **B** distante (par exemple Apogée) contenant les tables :
   * **UTILISATEUR**(int ID, str CODE, str NOM, str PRENOM, date NAISSANCE)
   * **COMPOSANTE**(int ID, str CODE, str NOM, int DIRECTEUR_ID, str ADRESSE)
   * **FORMATION**(int ID, str CODE, str NOM, str COMPOSANTE_CODE)
   * **CHOCOLATINE**(int ID, str NOM, str BOULANGERIE, int POURCENT_GRAS, int NOTE)
 
 
En outre, on suppose avoir déclaré les configurations _Doctrine_ 'orm_A' et 'orm_B' respectivement pour les bases **A** et **B** ci-dessus (soit dans le fichier [unicaen-db-import.local.php](config/unicaen-db-import.local.php.dist) soit dans un autre fichier de config.).

**Rappel :** L'ensemble des tables _Source_ et _Destination_ doivent exister au préalable.

Pour les exemples qui suivent, voici à quoi ressemble le fichier de config `unicaen-db-import.local.php` :

`unicaen-db-import.local.php`
```php
return [
    'import' => [
        'connections' => [
            'orm_A' => 'doctrine.connection.orm_A',
            'orm_B' => 'doctrine.connection.orm_B',
            'geo' => [
                'url'      => 'https://geo.api.gouv.fr/',
                'proxy'    => false,
                'verify'   => true,
                'user'     => null,
                'password' => null,
            ],
        ],
    ],
    'doctrine' => [
        'connection' => [
            'orm_A' => [
                'driverClass' => 'Doctrine\\DBAL\\Driver\\PDOPgSql\\Driver',
                'params' => [
                    'host'     => 'host.domain.fr',
                    'port'     => '5432',
                    'charset'  => 'utf8',
                    'user'     => '???',
                    'dbname'   => '???',
                    'password' => '???',
                ],
            ],
            'orm_B' => [
                'driverClass' => 'Doctrine\\DBAL\\Driver\\OCI8\\Driver',
                'params'      => [
                    'host'     => 'host.domain.fr',
                    'port'     => '1527',
                    'user'     => '???',
                    'password' => '???',
                    'dbname'   => '???',
                    'charset'  => 'AL32UTF8',
                ],
                'eventmanager' => 'orm_B',
            ],
        ],
];
```


### Exemple 1 : Import d'une table de données distantes

Dans de nombreux cas, on peut souhaiter importer une table de données provenant d'une base distante.
Ce peut être le cas notamment lorsque l'on souhaite s'assurer de la constante disponibilité des données.

`unicaen-db-import.global.php`
```php
return [
    'import' => [
        'imports' => [
            [
                'name' => "IMPORTATION DES DONNÉES B VERS A",
                'source' => [
                    'name'               => 'TABLE UTILISATEUR DE MA BASE B',
                    'table'              => 'UTILISATEUR',
                    'connection'         => 'orm_B',
                    'source_code_column' => 'CODE',
                ],
                'destination' => [
                    'name'               => 'TABLE UTILISATEUR DE MA BASE A',
                    'table'              => 'UTILISATEUR',
                    'connection'         => 'orm_A',
                    'source_code_column' => 'CODE',
                ],
            ],
        ],
    ],
];
```

`Terminal du serveur`
```bash
php public/index.php run import --name "IMPORTATION DES DONNÉES B VERS A"
```


### Exemple 2 : Import d'un select de données distantes

Les bases distantes requêtées sont généralement bien garnies.
Pourtant il est fréquent de ne vouloir importer qu'une partie de ces données.

`unicaen-db-import.global.php`
```php
return [
    'import' => [
        'imports' => [
            [
                'name' => "IMPORTATION PARTIELLE DES DONNÉES B VERS A",
                'source' => [
                    'name'               => 'TABLE CHOCOLATINE DE MA BASE B',
                    'select'             => 'SELECT ID, NOM, BOULANGERIE FROM CHOCOLATINE',
                    'connection'         => 'orm_B',
                    'source_code_column' => 'ID',
                ],
                'destination' => [
                    'name'               => 'TABLE PAIN_AU_CHOCOLAT DE MA BASE A',
                    'table'              => 'PAIN_AU_CHOCOLAT',
                    'connection'         => 'orm_A',
                    'source_code_column' => 'ID',
                ],
            ],
        ],
    ],
];
```

`Terminal du serveur`
```bash
php public/index.php run import --name "IMPORTATION PARTIELLE DES DONNÉES B VERS A"
```


### Exemple 3 : Import de données issues d'une API

Dans certains cas, on peut souhaiter importer les données provenant d'une API (web service).

`unicaen-db-import.global.php`
```php
return [
    'import' => [
        'imports' => [
            [
                'name' => "WS_IMPORT_REGIONS",
                'source' => [
                    'name'               => 'API Geo',
                    'select'             => '/regions?fields=nom,code',
                    'connection'         => 'geo',
                    'source_code_column' => 'code',
                ],
                'destination' => [
                    'name'               => 'Table',
                    'table'              => 'REGION',
                    'connection'         => 'orm_A',
                    'source_code_column' => 'CODE',
                ],
            ],
        ],
    ],
];
```

*NB: le `select` de ce type de source est la partie "requête" qui sera concaténée à l'`url` spécifiée
dans la config de la connexion.*

`Terminal du serveur`
```bash
php public/index.php run import --name "WS_IMPORT_REGIONS"
```


### Exemple 4 : Synchronisation de données locales

Dans la pratique, vous ne rencontrerez probablement pas ce cas d'exemple seul (voir [Exemple 5](#exemple-5-synchronisation-de-données-distantes-avec-récupération-des-clés-étrangères)).
Supposez donc posséder 2 tables, l'une d'elle pouvant être qualifiée de "brut"/"en vrac" (d'où proviennent les données) et l'autre "propre" (avec une historisation attendue).
On souhaite alors synchroniser la première table avec la seconde.

`unicaen-db-import.global.php`
```php
return [
    'import' => [
        'synchros' => [
            [
                'name' => "SYNCHRONISATION LOCALE DE DONNÉES DE A",
                'source' => [
                    'name'               => 'TABLE AUTRE_UTILISATEUR',
                    'select'             => 'SELECT CODE, NOM, PRENOM, NAISSANCE FROM AUTRE_UTILISATEUR',
                    'connection'         => 'orm_A',
                    'source_code_column' => 'CODE',
                ],
                'destination' => [
                    'name'               => 'TABLE UTILISATEUR',
                    'table'              => 'UTILISATEUR',
                    'connection'         => 'orm_A',
                    'source_code_column' => 'CODE',
                ],
            ],
        ],
    ],
];
```

`Terminal du serveur`
```bash
php public/index.php run synchro --name "SYNCHRONISATION LOCALE DE DONNÉES DE A"
```

*NB: Fonctionne également en spécifiant une 'table' au lieu d'un 'select' dans la 'source'.*


### Exemple 5 : Synchronisation de données distantes

#### Exemple 5.1 : Source de type base de données

Cet exemple sera probablement l'une des utilisations les plus récurrente d'UnicaenDbImport.
Par exemple, vous possédez une base distante contenant des utilisateurs et une base locale possédant ses propres utilisateurs.
Vous souhaitez alors harmoniser vos utilisateurs présent localement et ceux disponible sur la base distante.

`unicaen-db-import.global.php`
```php
return [
    'import' => [
        'synchros' => [
            [
                'name' => "SYNCHRONISATION DISTANTE DE DONNÉES DE B VERS A",
                'source' => [
                    'name'               => 'TABLE UTILISATEUR DE MA BASE B',
                    'select'             => 'SELECT CODE, NOM, PRENOM, NAISSANCE FROM UTILISATEUR',
                    'connection'         => 'orm_B',
                    'source_code_column' => 'CODE',
                ],
                'destination' => [
                    'name'               => 'TABLE UTILISATEUR DE MA BASE A',
                    'table'              => 'UTILISATEUR',
                    'connection'         => 'orm_A',
                    'source_code_column' => 'CODE',
                    
                    'intermediate_table' => 'UN_NOM_QUE_JE_FORCE', // sinon ce sera 'TMP_UTILISATEUR'
                    'intermediate_table_auto_drop' => true, // supprime la table temporaire si elle existe déjà

                ],
            ],
        ],
    ],
];
```

`Terminal du serveur`
```bash
php public/index.php run synchro --name "SYNCHRONISATION DISTANTE DE DONNÉES DE B VERS A"
```

*NB: Une synchronisation de données distantes fait en réalité appel à la fois au mécanisme d'import et au mécanisme de 
     synchronisation locale (voir [Fonctionnement](#fonctionnement)) ; d'où la présence (facultative) des paramètres 
     `intermediate_table` et `intermediate_table_auto_drop`.*

*NB-2: Fonctionne également en spécifiant une 'table' au lieu d'un 'select' dans la 'source'.*


#### Exemple 5.2 : Source de type API

`unicaen-db-import.global.php`
```php
return [
    'import' => [
        'synchros' => [
            [
                'name' => "WS_SYNCHRO_COMMUNES",
                'source' => [
                    'name'               => 'API Geo',
                    'select'             => '/departements/63/communes?fields=nom,code',
                    'connection'         => 'geo',
                    'source_code_column' => 'code',
                ],
                'destination' => [
                    'name'               => 'Table',
                    'table'              => 'REGION',
                    'connection'         => 'orm_A',
                    'source_code_column' => 'CODE',

                    'intermediate_table_auto_drop' => true,
                ],
            ],
        ],
    ],
];
```

`Terminal du serveur`
```bash
php public/index.php run synchro --name "WS_SYNCHRO_COMMUNES"
```


### Exemple 6 : Synchronisation de données distantes avec récupération des clés étrangères

De nombreux cas d'utilisation suivront cet exemple.
Si vous observez les tables (base locale) déclarées au début de cette section [Exemples](#exemples), vous pourrez constater que les formations possèdent des clés étrangères vers les composantes (via leur ID).
Pour gérer le cas de ces clés étrangères, il vous faudra :
- Déclarer un mécanisme d'import
- Déclarer un mécanisme de synchro
- Créer une vue (conventionnellement nommée SRC_*XXX*) réalisant les jointures nécessaires à la récupération des clés étrangères

`Console de la base de données locale`
```sql
CREATE VIEW SRC_FORMATION AS
    SELECT
        TMP_FORMATION.ID    AS ID,
        TMP_FORMATION.CODE  AS CODE,
        TMP_FORMATION.NOM   AS NOM,
        COMPOSANTE.ID       AS COMPOSANTE_ID
    FROM
        TMP_FORMATION
        INNER JOIN COMPOSANTE ON (TMP_FORMATION.COMPOSANTE_CODE = COMPOSANTE.CODE);
```

`unicaen-db-import.global.php`
```php
return [
    'import' => [
        'imports' => [
            [
                'name' => "IMPORTATION DE DONNÉES B VERS A",
                'source' => [
                    'name'               => 'TABLE FORMATION DE MA BASE B',
                    'select'             => 'SELECT ID, CODE, NOM, COMPOSANTE_CODE FROM FORMATION',
                    'connection'         => 'orm_B',
                    'source_code_column' => 'ID',
                ],
                'destination' => [
                    'name'               => 'TABLE TMP_FORMATION DE MA BASE A',
                    'table'              => 'TMP_FORMATION',
                    'connection'         => 'orm_A',
                    'source_code_column' => 'ID',
                ],
            ],
        ],
        'synchros' => [
            [
                'name' => "SYNCHRONISATION DE DONNÉES DÉJÀ IMPORTÉE DE B VERS A",   //Autrement dit une synchro locale
                'source' => [
                    'name'               => 'TABLE SRC_FORMATION DE MA BASE A UTILISANT TMP_FORMATION',
                    'select'             => 'SELECT * FROM SRC_UTILISATEUR',
                    'connection'         => 'orm_A',
                    'source_code_column' => 'ID',
                ],
                'destination' => [
                    'name'               => 'TABLE FORMATION DE MA BASE A',
                    'table'              => 'FORMATION',
                    'connection'         => 'orm_A',
                    'source_code_column' => 'ID',

                ],
            ],
        ],
    ],
];
```

`Terminal du serveur`
```bash
php public/index.php run import --name "IMPORTATION DE DONNÉES B VERS A"
php public/index.php run synchro --name "SYNCHRONISATION DE DONNÉES DÉJÀ IMPORTÉE DE B VERS A"
```

*NB: Dans cet exemple, la table TMP_XXX doit être préalablement créée. Elle ne sera donc pas supprimée à la fin de la synchronisation. De nouvelles fonctionnalités à venir devraient pouvoir automatiser cette suppression.*



Ajouter le support d'une autre plateforme de base de données (pour développeur)
-------------------------------------------------------------------------------

Imaginons que l'on veuille ajouter la possibilité d'importer/synchroniser vers la plateforme de base de données
destination MySQL.

- Ajouter dans le fichier `config/module.config.php` la config permettant d'associer la bonne classe de "code generator"
  (à créer) à chaque classe de plateforme de base de données MySQL connue de Doctrine :

```php
return [
    'import' => [
        'code_generators' => [
            \Doctrine\DBAL\Platforms\MySqlPlatform::class   => \UnicaenDbImport\CodeGenerator\MySQL\CodeGenerator::class,
            \Doctrine\DBAL\Platforms\MySQL57Platform::class => \UnicaenDbImport\CodeGenerator\MySQL\CodeGenerator::class,
            \Doctrine\DBAL\Platforms\MySQL80Platform::class => \UnicaenDbImport\CodeGenerator\MySQL\CodeGenerator::class,
        ],
        //...
```

- Créer un répertoire `src/UnicaenDbImport/CodeGenerator/MySQL` dans lequel on va créer les fichiers/classes 
  suivants :

```
.
├── CodeGeneratorFactory.php
├── CodeGenerator.php
├── Helper
│   ├── DiffViewHelper.php
│   ├── LogTableHelper.php
│   ├── TableHelper.php
│   └── TableValidationHelper.php
└── MySQLCommonsTrait.php
```

- La classe `CodeGenerator` doit *obligatoirement* :
  - hériter de la classe abstraite `\UnicaenDbImport\CodeGenerator\CodeGenerator` ;
  - instancier dans le constructeur ses propres versions MySQL des "helpers" de génération de code (`MySQLTableHelper`, etc.)

Exemple :

```php
namespace UnicaenDbImport\CodeGenerator\MySQL;

use Doctrine\DBAL\Platforms\AbstractPlatform;
use Doctrine\DBAL\Platforms\PostgreSqlPlatform;
use UnicaenDbImport\CodeGenerator\MySQL\Helper\DiffViewHelper as MySQLDiffViewHelper;
use UnicaenDbImport\CodeGenerator\MySQL\Helper\LogTableHelper as MySQLLogTableHelper;
use UnicaenDbImport\CodeGenerator\MySQL\Helper\TableValidationHelper as MySQLTableValidationHelper;
use UnicaenDbImport\CodeGenerator\MySQL\Helper\TableHelper as MySQLTableHelper;

/**
 * Version MySQL.
 */
class CodeGenerator extends \UnicaenDbImport\CodeGenerator\CodeGenerator
{
    /**
     * @var PostgreSqlPlatform
     */
    protected $platform;

    /**
     * @var MySQLTableHelper
     */
    protected $tableHelper;

    /**
     * @var MySQLTableValidationHelper
     */
    protected $tableValidationHelper;

    /**
     * @var MySQLDiffViewHelper
     */
    protected $diffViewHelper;

    /**
     * @var MySQLLogTableHelper
     */
    protected $synchroLogHelper;

    /**
     * CodeGenerator constructor.
     *
     * @param AbstractPlatform $platform
     */
    public function __construct(AbstractPlatform $platform)
    {
        parent::__construct($platform);

        $this->tableHelper = new MySQLTableHelper($this->platform);
        $this->tableValidationHelper = new MySQLTableValidationHelper($this->platform);
        $this->diffViewHelper = new MySQLDiffViewHelper($this->platform);
        $this->synchroLogHelper = new MySQLLogTableHelper($this->platform);
    }
    
    //...
```

- Sa factory `CodeGeneratorFactory` doit *obligatoirement* :
  - injecter dans le "code generator" une instance de la plateforme correspondant à la base de données destination.
  
    *NB: l'instance de la plateforme de base de données injectée ici est utilisée pour générer du SQL compris par 
    la base de données destination. Si aucune classe de platforme ne correspond exactement à la version de la base 
    de données destination, prenez la générique (`MySqlPlatform` dans notre exemple)*

Exemple :

```php
namespace UnicaenDbImport\CodeGenerator\MySQL;

use Doctrine\DBAL\Platforms\MySqlPlatform;
use Interop\Container\ContainerInterface;
use UnicaenDbImport\CodeGenerator\MySQL\CodeGenerator as MySQLCodeGenerator;

class CodeGeneratorFactory
{
    /**
     * @param ContainerInterface $container
     * @return CodeGenerator
     */
    public function __invoke(ContainerInterface $container)
    {
        return new MySQLCodeGenerator(new MySqlPlatform());
    }
}
```


- La classe de "helper" de génération de code `Helper\DiffViewHelper` doit :
  - *obligatoirement* hériter de la classe abstraite `\UnicaenDbImport\CodeGenerator\Helper\DiffViewHelper` ;
  - *obligatoirement* définir les méthodes non implémentées par la classe mère ;
  - *si besoin* redéfinir les méthodes de la classe mère qui ne génèreraient pas du code SQL valide pour la
    plateforme de base de données (ici, MySQL).

Exemple :

```php
namespace UnicaenDbImport\CodeGenerator\MySQL\Helper;

use Doctrine\DBAL\Platforms\MySqlPlatform;
use UnicaenDbImport\Domain\Operation;

class DiffViewHelper extends \UnicaenDbImport\CodeGenerator\Helper\DiffViewHelper
{
    /**
     * @var MySqlPlatform
     */
    protected $platform;

    /**
     * {@inheritDoc}
     */
    protected function generateViewDeletionSQLSnippet($destinationTable)
    {
        $name = $this->generateViewName($destinationTable);

        return "DROP VIEW IF EXISTS $name";
    }
  
    //...
```


- Mêmes principes pour les autres classes `Helper\LogTableHelper`, `Helper\TableHelper`, `Helper\TableValidationHelper`.


- Du fait que les classes `CodeGenerator` et `*Helper` héritent déjà chacune d'une classe, un trait `MySQLCommonsTrait` 
  peut être utile pour partager des éléments identiques nécessaires dans toutes ces classes.
  
Exemple pour la plateforme PostgreSQL :

```php
namespace UnicaenDbImport\CodeGenerator\PostgreSQL;

trait PostgreSQLCommonsTrait
{
    /**
     * @param string $argument
     * @return string
     */
    protected function getQuoteLiteralFunctionCallSQLSnippet($argument)
    {
        return 'quote_literal(' . $argument . ')';
    }
}
```
