<?php

namespace UnicaenDbImport;

use Doctrine\DBAL\Platforms\PostgreSQL100Platform;
use Doctrine\DBAL\Platforms\PostgreSQL91Platform;
use Doctrine\DBAL\Platforms\PostgreSQL92Platform;
use Doctrine\DBAL\Platforms\PostgreSQL94Platform;
use Doctrine\DBAL\Platforms\PostgreSqlPlatform;
use UnicaenDbImport\CodeGenerator\PostgreSQL\CodeGenerator as PostgreSQLCodeGenerator;
use UnicaenDbImport\Config\Config;
use UnicaenDbImport\Config\ConfigFactory;
use UnicaenDbImport\Controller\ConsoleControllerFactory;
use UnicaenDbImport\Service\ApiService;
use UnicaenDbImport\Service\ApiServiceFactory;
use UnicaenDbImport\Service\CodeGeneratorPluginManager;
use UnicaenDbImport\Service\CodeGeneratorPluginManagerFactory;
use UnicaenDbImport\Service\DatabaseService;
use UnicaenDbImport\Service\DatabaseServiceFactory;
use UnicaenDbImport\Service\FacadeService;
use UnicaenDbImport\Service\FacadeServiceFactory;
use UnicaenDbImport\Service\ImportService;
use UnicaenDbImport\Service\ImportServiceFactory;
use UnicaenDbImport\Service\SynchroService;
use UnicaenDbImport\Service\SynchroServiceFactory;

return [
    'import' => [
        'connections' => [],
        'code_generators' => [
            PostgreSqlPlatform::class    => PostgreSQLCodeGenerator::class,
            PostgreSQL91Platform::class  => PostgreSQLCodeGenerator::class,
            PostgreSQL92Platform::class  => PostgreSQLCodeGenerator::class,
            PostgreSQL94Platform::class  => PostgreSQLCodeGenerator::class,
            PostgreSQL100Platform::class => PostgreSQLCodeGenerator::class,
        ],
        'imports' => [],
        'synchros' => [],
    ],

    'console' => [
        'router' => [
            'routes' => [
                'execute_imports' => [
                    'type'    => 'simple',
                    'options' => [
                        'route'    => 'run import [--all|-a] [--name=]',
                        'defaults' => [
                            'controller' => 'UnicaenDbImport\Controller\Console',
                            'action'     => 'runImport',
                        ],
                    ],
                ],
                'execute_synchro' => [
                    'type'    => 'simple',
                    'options' => [
                        'route'    => 'run synchro [--all|-a] [--name=]',
                        'defaults' => [
                            'controller' => 'UnicaenDbImport\Controller\Console',
                            'action'     => 'runSynchro',
                        ],
                    ],
                ],
            ]
        ]
    ],

    'controllers' => [
        'factories' => [
            'UnicaenDbImport\Controller\Console' => ConsoleControllerFactory::class,
        ]
    ],

    'service_manager' => [
        'factories' => [
            Config::class => ConfigFactory::class,
            ImportService::class => ImportServiceFactory::class,
            SynchroService::class => SynchroServiceFactory::class,
            FacadeService::class => FacadeServiceFactory::class,
            DatabaseService::class => DatabaseServiceFactory::class,
            ApiService::class => ApiServiceFactory::class,
            CodeGeneratorPluginManager::class => CodeGeneratorPluginManagerFactory::class,
        ],
    ],
];
