<?php

namespace UnicaenDbImport\CodeGenerator\Helper;

use Doctrine\DBAL\DBALException;
use UnicaenApp\Exception\RuntimeException;
use UnicaenDbImport\CodeGenerator\Helper;
use UnicaenDbImport\Domain\ImportResult;
use UnicaenDbImport\Domain\ResultInterface;
use UnicaenDbImport\Domain\SynchroResult;

/**
 * Génération de code SQL concernant la table de log de synchro.
 *
 * Version commune à toutes les plateformes de bases de données.
 *
 * @author Unicaen
 */
abstract class LogTableHelper extends Helper
{
    /**
     * @param string $logTableName
     * @return string
     */
    public function generateSQLForLogTableCreation(string $logTableName)
    {
        $varchar = $this->platform->getVarcharTypeDeclarationSQL(['length' => 128]);
        $smallint = $this->platform->getSmallIntTypeDeclarationSQL([]);
        $text = $this->platform->getClobTypeDeclarationSQL([]);
        try {
            $datetime = $this->platform->getDateTimeTypeDeclarationSQL([]);
        } catch (DBALException $e) {
            throw new RuntimeException("Non implémentée!");
        }

        $sql = <<<EOT
CREATE TABLE $logTableName (
    type $varchar not null,
    name $varchar not null,
    success $smallint not null,
    log $text not null,
    started_on $datetime not null,
    ended_on   $datetime not null
);
EOT;
        return $sql;
    }

    /**
     * @param ResultInterface $result
     * @param string          $logTableName
     * @return string
     */
    public function generateSQLForInsertResultIntoLogTable(ResultInterface $result, string $logTableName)
    {
        switch (true) {
            case $result instanceof ImportResult:
                $type = 'import';
                $name = $result->getImport()->getName();
                break;
            case $result instanceof SynchroResult:
                $type = 'synchro';
                $name = $result->getSynchro()->getName();
                break;
            default:
                throw new RuntimeException("Type de résultat inattendu");
                break;
        }
        $success = $result->getException() ? 0 : 1;
        $startDate = $result->getStartDate()->format("Y-m-d H:i:s");
        $endDate = $result->getEndDate()->format("Y-m-d H:i:s");

        $sql = '';
        $sql .= "INSERT INTO $logTableName (type, name, success, log, started_on, ended_on) ";
        $sql .= sprintf("VALUES (%s, %s, %s, %s, %s, %s)",
            $this->platform->quoteStringLiteral($type),
            $this->platform->quoteStringLiteral($name),
            $success,
            $this->platform->quoteStringLiteral($result->toString(true)),
            $this->platform->quoteStringLiteral($startDate),
            $this->platform->quoteStringLiteral($endDate)
        );

        return $sql;
    }
}