<?php

namespace UnicaenDbImport\CodeGenerator\Helper;

use DateTime;
use DateTimeZone;
use stdClass;
use UnicaenApp\Exception\RuntimeException;
use UnicaenApp\Util;
use UnicaenDbImport\CodeGenerator\Helper;

/**
 * Génération de code SQL autour des tables.
 *
 * Version commune à toutes les plateformes de bases de données.
 *
 * @author Unicaen
 */
abstract class TableHelper extends Helper
{
    /**
     * @param string $tableName
     * @return string
     */
    public function generateSQLForSelectFromTable($tableName)
    {
        return "SELECT * FROM " . $tableName;
    }

    /**
     * @param string   $destinationTable
     * @param string   $sourceCodeColumn
     * @param string[] $columns
     * @param string   $intermediateTable
     * @return string
     */
    public function generateSQLForIntermediateTableCreation($destinationTable, $sourceCodeColumn, $columns, $intermediateTable)
    {
        $commaSeparatedColumnNames = implode(', ', array_merge([$sourceCodeColumn], $columns));

        $sqlTemplate = <<<'EOT'
create table {intermediateTable} as select {commaSeparatedColumnNames} from {destinationTable};
delete from {intermediateTable};
EOT;
        return Util::tokenReplacedString($sqlTemplate, compact('intermediateTable', 'destinationTable', 'commaSeparatedColumnNames'));
    }

    /**
     * @param string $tableName
     * @param array  $columnsValues
     * @return string
     */
    public function generateSQLForInsertIntoTable($tableName, array $columnsValues)
    {
        $platform = $this->platform;

        $formatter = function ($value) use ($platform) {
            if ($value === null) {
                return 'NULL';
            }

            if ($value instanceof stdClass) {
                if (isset($value->date)) {
                    $tz = isset($value->timezone) ? new DateTimeZone($value->timezone) : NULL;
                    $d = new DateTime($value->date, $tz);
                    $value = $d->format('c');
                } else {
                    throw new RuntimeException("Type de valeur inattendu!");
                }
            }

            return $platform->quoteStringLiteral($value);
        };

        $commaSeparatedColumnNames = implode(', ', array_keys($columnsValues));
        $commaSeparatedColumnValues = implode(', ', array_map($formatter, $columnsValues));

        $sqlTemplate = <<<'EOT'
insert into {tableName} ({commaSeparatedColumnNames}) values ({commaSeparatedColumnValues});
EOT;
        return Util::tokenReplacedString($sqlTemplate, compact('tableName', 'commaSeparatedColumnNames', 'commaSeparatedColumnValues'));
    }

    /**
     * @param string $tableName
     * @return string
     */
    public function generateSQLForTruncateTable($tableName)
    {
        return $this->platform->getTruncateTableSQL($tableName);
    }
}