<?php

namespace UnicaenDbImport\CodeGenerator\PostgreSQL\Helper;

use Doctrine\DBAL\Platforms\PostgreSqlPlatform;
use UnicaenDbImport\CodeGenerator\PostgreSQL\PostgreSQLCommonsTrait;
use UnicaenDbImport\Domain\Operation;

class DiffViewHelper extends \UnicaenDbImport\CodeGenerator\Helper\DiffViewHelper
{
    use PostgreSQLCommonsTrait;

    /**
     * @var PostgreSqlPlatform
     */
    protected $platform;

    /**
     * {@inheritDoc}
     */
    protected function generateViewDeletionSQLSnippet($destinationTable)
    {
        $name = $this->generateViewName($destinationTable);

        return "DROP VIEW IF EXISTS $name";
    }

    /**
     * {@inheritDoc}
     */
    protected function generateSQLForUpdateOperationInDestinationTable($destinationTable, $sourceCodeColumn, array $columns)
    {
        $diffViewName = $this->generateViewName($destinationTable);

        $now = $this->platform->getNowExpression();

        $colUpdates = array_map(function ($col) {
            return "$col = diff.S_$col";
        }, $columns);

        $update = Operation::OPERATION_UPDATE;

        // instruction de mise à jour de la table destination
        $markup = '';
        $markup .= "UPDATE $destinationTable d SET " . implode(', ', $colUpdates) . ", updated_on = $now, deleted_on = null " . PHP_EOL;
        $markup .= "FROM (SELECT * FROM $diffViewName) AS diff " . PHP_EOL;
        $markup .= "WHERE diff.operation = '$update' AND d.$sourceCodeColumn = diff.$sourceCodeColumn ";

        return $markup;
    }

    /**
     * {@inheritDoc}
     */
    protected function generateSQLForUndeleteOperationInDestinationTable($destinationTable, $sourceCodeColumn, array $columns)
    {
        $diffViewName = $this->generateViewName($destinationTable);

        $now = $this->platform->getNowExpression();

        $colUpdates = array_map(function ($col) {
            return "$col = diff.S_$col";
        }, $columns);

        $undelete = Operation::OPERATION_UNDELETE;

        // instruction de mise à jour de la table destination
        $markup = '';
        $markup .= "UPDATE $destinationTable d SET " . implode(', ', $colUpdates) . ", updated_on = $now, deleted_on = null " . PHP_EOL;
        $markup .= "FROM (SELECT * FROM $diffViewName) AS diff " . PHP_EOL;
        $markup .= "WHERE diff.operation = '$undelete' AND d.$sourceCodeColumn = diff.$sourceCodeColumn ";

        return $markup;
    }

    /**
     * {@inheritDoc}
     */
    protected function generateSQLForDeleteOperationInDestinationTable($destinationTable, $sourceCodeColumn, array $columns)
    {
        $diffViewName = $this->generateViewName($destinationTable);

        $now = $this->platform->getNowExpression();

        $delete = Operation::OPERATION_DELETE;

        // instruction de mise à jour de la table destination
        $markup = '';
        $markup .= "UPDATE $destinationTable d SET deleted_on = $now" . PHP_EOL;
        $markup .= "FROM (SELECT operation, $sourceCodeColumn FROM $diffViewName) AS diff " . PHP_EOL;
        $markup .= "WHERE diff.operation = '$delete' AND d.$sourceCodeColumn = diff.$sourceCodeColumn ";

        return $markup;
    }
}