<?php

namespace UnicaenDbImport\CodeGenerator\PostgreSQL\Helper;

use Doctrine\DBAL\Platforms\PostgreSqlPlatform;
use UnicaenDbImport\CodeGenerator\PostgreSQL\PostgreSQLCommonsTrait;

/**
 * Version PostgreSQL.
 *
 * @author Unicaen
 */
class TableValidationHelper extends \UnicaenDbImport\CodeGenerator\Helper\TableValidationHelper
{
    use PostgreSQLCommonsTrait;

    /**
     * @var PostgreSqlPlatform
     */
    protected $platform;

    /**
     * {@inheritDoc}
     */
    public function generateSQLForTableExistenceCheck($tableName)
    {
        $sql = <<<EOT
SELECT count(*) as table_exists
FROM   information_schema.tables
WHERE  table_schema = 'public' AND upper(table_name) = upper('$tableName')
;
EOT;
        return $sql;
    }

    /**
     * {@inheritDoc}
     */
    public function generateSQLForColumnsValidation($tableName, array $columnsAndTypes)
    {
        $selects = [];
        foreach ($columnsAndTypes as $column => $type) {
            if (is_numeric($column)) {
                $column = $type;
                $type = 'NULL';
            } else {
                $type = "'$type'";
            }
            $selects[] = "SELECT '$column', $type";
        }
        $selects = implode(' UNION' . PHP_EOL, $selects);

        $sql = <<<EOT

WITH required_cols(column_name, column_type) AS (
$selects
)
SELECT rc.column_name, c.data_type
FROM required_cols rc
JOIN information_schema.columns c ON upper(c.column_name) = upper(rc.column_name)
WHERE upper(c.table_name) = upper('$tableName') AND (rc.column_type  IS NULL OR rc.column_type = c.data_type)
;
EOT;
        return $sql;
    }

    /**
     * {@inheritDoc}
     */
    public function generateSQLForHistoColumnsValidation($tableName)
    {
        $columnsAndTypes = [
            'created_on' => 'timestamp without time zone',
            'updated_on' => 'timestamp without time zone',
            'deleted_on' => 'timestamp without time zone',
        ];

        return $this->generateSQLForColumnsValidation($tableName, $columnsAndTypes);
    }

    /**
     * {@inheritDoc}
     */
    public function generateSQLForSourceColumnsValidation($tableName)
    {
        $columnsAndTypes = [
            'source_code' => 'character varying',
            'source_id' => 'integer',
        ];

        return $this->generateSQLForColumnsValidation($tableName, $columnsAndTypes);
    }
}