<?php

namespace UnicaenDbImport\Config;

use UnicaenDbImport\Domain\ImportInterface;
use UnicaenDbImport\Domain\SynchroInterface;

/**
 * Config du module.
 *
 * Permet la récupération des instances Import, Synchro et Importers.
 */
class Config
{
    /**
     * platform's class => code generator's class
     * @var array
     */
    protected $codeGeneratorsConfig = [];

    /**
     * @var ImportInterface[]
     */
    protected $imports = [];

    /**
     * @var SynchroInterface[]
     */
    protected $synchros = [];

    /**
     * @param array $codeGeneratorsConfig
     * @return Config
     */
    public function setCodeGeneratorsConfig(array $codeGeneratorsConfig): Config
    {
        $this->codeGeneratorsConfig = $codeGeneratorsConfig;

        return $this;
    }

    /**
     * @return array
     */
    public function getCodeGeneratorsConfig(): array
    {
        return $this->codeGeneratorsConfig;
    }

    /**
     * @param ImportInterface[] $imports
     * @return static
     */
    public function setImports(array $imports)
    {
        $this->imports = $imports;

        return $this;
    }

    /**
     * @return ImportInterface[]
     */
    public function getImports()
    {
        return $this->imports;
    }

    /**
     * @param string $name
     * @return ImportInterface
     */
    public function getImport($name): ?ImportInterface
    {
        foreach ($this->imports as $import) {
            if ($import->getName() === $name) {
                return $import;
            }
        }

        return null;
    }

    /**
     * @param SynchroInterface[] $synchros
     * @return static
     */
    public function setSynchros(array $synchros): self
    {
        $this->synchros = $synchros;

        return $this;
    }

    /**
     * @return SynchroInterface[]
     */
    public function getSynchros(): array
    {
        return $this->synchros;
    }

    /**
     * @param string $name
     * @return SynchroInterface
     */
    public function getSynchro($name): ?SynchroInterface
    {
        foreach ($this->synchros as $synchro) {
            if ($synchro->getName() === $name) {
                return $synchro;
            }
        }

        return null;
    }
}