<?php

namespace UnicaenDbImport\Connection;

use Zend\Hydrator\ClassMethods;

class ApiConnection
{
    /**
     * @var string
     */
    private $url;
    /**
     * @var boolean
     */
    private $proxy = false;
    /**
     * @var boolean
     */
    private $verify = false;
    /**
     * @var string
     */
    private $user;
    /**
     * @var string
     */
    private $password;

    /**
     * @var string
     */
    private $pageParam = 'page';

    /**
     * @var string
     */
    private $pageSizeParam = 'page_size';

    /**
     * @param array $config
     * @return ApiConnection
     */
    public static function fromArrayConfig(array $config)
    {
        $inst = new static();

        $h = new ClassMethods();
        $h->hydrate($config, $inst);

        return $inst;
    }

    /**
     * @return array
     */
    public function toArray(): array
    {
        $h = new ClassMethods();

        return $h->extract($this);
    }

    /**
     * @return string
     */
    public function getUrl(): string
    {
        return $this->url;
    }

    /**
     * @param string $url
     * @return ApiConnection
     */
    public function setUrl(string $url): ApiConnection
    {
        $this->url = $url;
        return $this;
    }

    /**
     * @return bool
     */
    public function getProxy(): bool
    {
        return $this->proxy;
    }

    /**
     * @param bool $proxy
     * @return ApiConnection
     */
    public function setProxy(bool $proxy = true): ApiConnection
    {
        $this->proxy = $proxy;
        return $this;
    }

    /**
     * @return bool
     */
    public function getVerify(): bool
    {
        return $this->verify;
    }

    /**
     * @param bool $verify
     * @return ApiConnection
     */
    public function setVerify(bool $verify = true): ApiConnection
    {
        $this->verify = $verify;
        return $this;
    }

    /**
     * @return string
     */
    public function getUser(): ?string
    {
        return $this->user;
    }

    /**
     * @param string $user
     * @return ApiConnection
     */
    public function setUser(string $user = null): ApiConnection
    {
        $this->user = $user;
        return $this;
    }

    /**
     * @return string
     */
    public function getPassword(): ?string
    {
        return $this->password;
    }

    /**
     * @param string $password
     * @return ApiConnection
     */
    public function setPassword(string $password = null): ApiConnection
    {
        $this->password = $password;
        return $this;
    }

    /**
     * @return string
     */
    public function getPageParam(): string
    {
        return $this->pageParam;
    }

    /**
     * @param string $pageParam
     * @return ApiConnection
     */
    public function setPageParam(string $pageParam): ApiConnection
    {
        $this->pageParam = $pageParam;

        return $this;
    }

    /**
     * @return string
     */
    public function getPageSizeParam(): string
    {
        return $this->pageSizeParam;
    }

    /**
     * @param string $pageSizeParam
     * @return ApiConnection
     */
    public function setPageSizeParam(string $pageSizeParam): ApiConnection
    {
        $this->pageSizeParam = $pageSizeParam;

        return $this;
    }
}