<?php

namespace UnicaenDbImport\Domain;

use Zend\Filter\FilterChain;
use Zend\Filter\FilterInterface;
use Zend\Filter\StringToUpper;
use Zend\Filter\Word\CamelCaseToUnderscore;

/**
 * Filtre de transformation par défaut d'un nom de colonne/attribut issu de la source
 * en le nom de colonne destination.
 *
 * Exemples :
 * - 'dateDebValidite'     => 'DATE_DEB_VALIDITE'
 * - 'date_debut_validite' => 'DATE_DEB_VALIDITE'
 * - 'DATE_DEB_VALIDITE'   => 'DATE_DEB_VALIDITE'
 *
 * @author Unicaen
 */
class ColumnNameFilter implements FilterInterface
{
    /**
     * Returns the result of filtering $value
     *
     * @param string $columnName
     * @return string
     */
    public function filter($columnName)
    {
        $chain = new FilterChain();

        // dateDebValidite => date_Deb_Validite
        $chain->attach(new CamelCaseToUnderscore());

        // date_Deb_Validite => DATE_DEB_VALIDITE
        $chain->attach(new StringToUpper());

        return $chain->filter($columnName);
    }
}