<?php

namespace UnicaenDbImport\Domain;

use Doctrine\DBAL\Connection;

interface DestinationInterface
{
    const LOG_TABLE_DEFAULT_NAME = 'import_log';

    const HISTO_COLUMNS = ['CREATED_ON', 'UPDATED_ON', 'DELETED_ON'];

    /**
     * Retourne le petit nom de cette destination.
     *
     * @return string
     */
    public function getName();

    /**
     * Retourne le nom de la table de destination.
     *
     * @return string
     */
    public function getTable();

    /**
     * Retourne l'instance de la connexion Doctrine vers cette destination.
     *
     * @return Connection
     */
    public function getConnection();

    /**
     * Retourne le nom de la colonne commune existant obligatoirement dans la table/vue source
     * ET la table destination.
     *
     * Dans les tables source et destination, cette colonne est celle d'un identifiant unique permettant
     * de rapprocher les enregistrements source et destination.
     *
     * @return string Exemple: 'code'
     */
    public function getSourceCodeColumn();

    /**
     * Retourne les noms des colonnes de la table destination concernées par l'import.
     *
     * Ne doit pas inclure la colonne commune retournée par {@link getSourceCodeColumn}.
     *
     * @return array Exemple: ['libelle', 'debut_validite', 'fin_validite']
     */
    public function getColumns();


    /**
     * Retourne le nom de la table intermédiaire à créer dans la base de données de destination
     * lorsque l'utilisation d'une table intermédiaire est nécessaire (cf {@link requiresIntermediateTable}).
     *
     * @return string
     */
    public function getIntermediateTable();

    /**
     * Indique si la suppression automatique de la table intermédiaire est activée.
     *
     * @return bool
     */
    public function getIntermediateTableAutoDrop();

    /**
     * Retourne le nom de la table des logs.
     *
     * @return string
     */
    public function getLogTable();

}