<?php

namespace UnicaenDbImport\Domain;

use Assert\AssertionChain;
use Exception;
use UnicaenDbImport\Config\ConfigException;
use Zend\Stdlib\Parameters;

class Import implements ImportInterface
{
    /**
     * @var Parameters
     */
    protected $config;

    /**
     * @param array $config
     * @return static
     */
    static public function fromConfig(array $config)
    {
        $inst = new static($config);

        try {
            $inst->validateConfig();
        } catch (Exception $e) {
            if (! $e instanceof ConfigException) {
                $e = new ConfigException($e->getMessage(), 0, $e);
            }
            throw $e;
        }

        return $inst;
    }

    protected function __construct(array $config)
    {
        $this->config = new Parameters($config);
    }

    protected function validateConfig()
    {
        (new AssertionChain($this->config->get($key = 'source'), "Une instance de " . SourceInterface::class . " est requise pour la clé suivante: $key"))
            ->notEmpty()
            ->isInstanceOf(SourceInterface::class);

        (new AssertionChain($this->config->get($key = 'destination'), "Une instance de " . DestinationInterface::class . " est requise pour la clé suivante: $key"))
            ->notEmpty()
            ->isInstanceOf(DestinationInterface::class);
    }

    /**
     * @return string
     */
    public function __toString()
    {
        return $this->getName();
    }

    /**
     * @return string
     */
    public function getName()
    {
        if ($name = $this->config->get('name')) {
            return $name;
        }

        return $this->getSource()->getName() . ' ==> ' . $this->getDestination()->getName();
    }

    /**
     * @return Source
     */
    public function getSource()
    {
        return $this->config->get('source');
    }

    /**
     * @return Destination
     */
    public function getDestination()
    {
        return $this->config->get('destination');
    }
}