<?php

namespace UnicaenDbImport\Domain;

use DateTime;
use Exception;

abstract class Result implements ResultInterface
{
    /**
     * @var Exception
     */
    private $exception;

    /**
     * @var array
     */
    private $resultsByOperation = [];

    /**
     * @var DateTime
     */
    private $startDate;

    /**
     * @var DateTime
     */
    private $endDate;

    /**
     * @param array $data
     * @param string $operation
     */
    public function setResultsForOperation(array $data, $operation)
    {
        $this->resultsByOperation[$operation] = $data;
    }

    /**
     * @return Exception
     */
    public function getException()
    {
        return $this->exception;
    }

    /**
     * @param Exception $exception
     */
    public function setException(Exception $exception = null)
    {
        $this->exception = $exception;
    }

    /**
     * @return DateTime
     */
    public function getStartDate(): DateTime
    {
        return $this->startDate;
    }

    /**
     * @param DateTime $startDate
     */
    public function setStartDate(DateTime $startDate = null)
    {
        $this->startDate = $startDate ?: date_create();
    }

    /**
     * @return DateTime
     */
    public function getEndDate(): DateTime
    {
        return $this->endDate;
    }

    /**
     * @param DateTime $endDate
     */
    public function setEndDate(DateTime $endDate = null)
    {
        $this->endDate = $endDate ?: date_create();
    }

    /**
     * @param bool $includeDetails
     * @return string
     */
    public function toString($includeDetails = false)
    {
        if ($ex = $this->getException()) {
            return $ex->getMessage() . PHP_EOL . $ex->getTraceAsString();
        }

        if (count($this->resultsByOperation) === 0) {
            return "Aucune instruction exécutée.";
        }

        $str = '';
        foreach ($this->resultsByOperation as $operation => $results) {
            $str .= "# " . $operation . " : " . PHP_EOL;
            if ($includeDetails && isset($results['details'])) {
                $str .= $results['details'] . PHP_EOL;
            }
            $str .= $results['count'] . " instructions exécutées.";
            $str .= PHP_EOL;
        }

        return $str;
    }

    /**
     * @return string
     */
    public function __toString()
    {
        return $this->toString();
    }
}