<?php

namespace UnicaenDbImport;

use Doctrine\DBAL\Connection;
use Doctrine\DBAL\DBALException;
use Doctrine\DBAL\Driver\ResultStatement;
use Doctrine\DBAL\Query\QueryBuilder;
use UnicaenApp\Exception\RuntimeException;

/**
 * @codeCoverageIgnore
 *
 * @author Unicaen
 */
class QueryExecutor
{
    /**
     * @param string     $sql
     * @param Connection $connection
     * @return ResultStatement
     */
    private function executeQuery($sql, Connection $connection)
    {
        try {
            return $connection->executeQuery($sql);
        } catch (DBALException $e) {
            throw new RuntimeException("Erreur rencontrée lors de l'exécution de la requête", null, $e);
        }
    }

    /**
     * @param QueryBuilder $qb
     * @param int          $firstResult
     * @param int          $maxResults
     * @return array
     */
    public function fetchPartial(QueryBuilder $qb, $firstResult, $maxResults)
    {
        //
        // Attention, brouillon!
        //

        $qb
            ->setFirstResult($firstResult)
            ->setMaxResults($maxResults);

        $statement = $qb->execute();

        return $statement->fetchAll(\PDO::FETCH_ASSOC);
    }

    /**
     * @param string     $sql
     * @param Connection $connection
     * @return array
     */
    public function fetchAll($sql, Connection $connection)
    {
        $statement = $this->executeQuery($sql, $connection);

        return $statement->fetchAll(\PDO::FETCH_ASSOC);
    }

    /**
     * @param string     $sql
     * @param Connection $connection
     * @return int
     */
    public function exec($sql, Connection $connection)
    {
        try {
            return $connection->exec($sql);
        } catch (DBALException $e) {
            throw new RuntimeException("Erreur rencontrée lors de l'exécution de la requête", null, $e);
        }
    }
}